describe NaturalBornSlugger::AttributeComposer do

  context 'class methods' do
    subject(:composer){ NaturalBornSlugger::AttributeComposer }

    describe ".default_options" do
      it "should not require that all dependent attributes be present to form composite" do
        composer.default_options[:require_all].should be_false
      end
      it "should not join depencencies with a special character" do
        composer.default_options[:join_with].should be_empty
      end
      it "should not use a callback" do
        composer.default_options[:callback].should be_false
      end
    end

    describe ".extract_dependencies_from_options" do
      let(:empty_opts) { {} }
      let(:simple_opts) { {name: 'foobar', title: 'asdf'} }
      let(:config_opts) { {require_all: true, join_with: '^'} }
      let(:mixed_opts) { {name: 'foobar', title: 'asdf', require_all: true, join_with: '^'} }
      it "should always return 3 options" do
        [empty_opts, simple_opts, config_opts, mixed_opts].each do |options|
          _, opts = composer.extract_dependencies_from_options(options)
          opts.length.should eql composer.default_options.length
        end
      end
    end

    describe ".resolve_dependency" do
      let(:example_object) { Example.new }
      it "should retrieve attributes listed as a dependency" do
        composer.resolve_dependency(example_object, 'attribute').should == 'attribute'
      end
      it "should use the results of methods listed as a dependency" do
        composer.resolve_dependency(example_object, 'meth').should == 'method'
      end
      it "should accept methods that return objects as a dependency" do
        composer.resolve_dependency(example_object, 'child').should be_an ExampleChild
      end
      context "through related objects via dot syntax" do
        it "should resolve a related object's attributes" do
          composer.resolve_dependency(example_object, 'child.attribute').should == 'child attribute'
        end
        it "should resolve a related object's methods" do
          composer.resolve_dependency(example_object, 'child.meth').should == 'child method'
        end
        it "should resolve a related object's own related objects" do
          composer.resolve_dependency(example_object, 'child.child').should be_an ExampleGrandChild
        end
      end
    end

  end


  context 'instance methods' do
    context '#add_to' do
      subject (:target) { FreezableObject.new }
      describe 'an object with a composite attribute' do
        it 'should have a method named after the composite attribute' do
          target.respond_to?(:natural_key).should be_true
        end
        it 'should have a method used to update the attribute' do
          target.respond_to?(:update_natural_key).should be_true
        end
      end
      describe 'a frozen object' do
        it 'should not auto update composite attributes' do
          target.freeze
          expect { target }.not_to receive(:update_natural_key)
          target.natural_key
        end
      end
    end
    describe '#compose_attribute' do

    end

    describe '#evaluate' do

    end

  end




end
